@@ -1,64 +1,7 @@
-#' mgwrsar Model Predictions
-#' predict_mgwrsar is a function for computing predictions of a mgwrsar models. It uses Best Linear Unbiased Predictor for mgwrsar models with spatial autocorrelation.
-#' @usage predict_mgwrsar(model, newdata, newdata_coords, W = NULL,
-#' type = "BPN", h_w = 100,kernel_w = "rectangle",maxobs=4000,
-#' beta_proj=FALSE,method_pred='TP', k_extra = 8,exposant=6)
-#' @param model a model of mgwrsar class.
-#' @param newdata a matrix or data.frame of new data.
-#' @param newdata_coords  a matrix of new coordinates, and eventually other variables if a General Kernel Product is used.
-#' @param W the spatial weight matrix for models with  spatial autocorrelation.
-#' @param type Type for BLUP estimator, default "BPN". If NULL use predictions without spatial bias correction.
-#' @param  h_w A bandwidth value for the spatial weight matrix
-#' @param kernel_w kernel type for the spatial weight matrix. Possible types:
-#' rectangle ("rectangle"), bisquare ("bisq"), tricube ("tcub"),
-#' epanechnikov ("epane"), gaussian ("gauss")) .
-#' @param maxobs  maximum number of observations for exact calculation of solve(I- rho*W), default maxobs=4000.
-#' @param beta_proj A boolean, if TRUE the function then return a two elements list(Y_predicted,Beta_proj_out)
-#' @param method_pred If method_pred = 'TP' (default) prediction is done by recomputing a MGWRSAR model
-#' with new-data as target points, else if method_pred in ('tWtp_model','model','shepard') a matrix
-#' for projecting estimated betas is used (see details).
-#' @param k_extra number of neighboors for local parameter extrapolation if shepard kernel is used, default 8.
-#' @param exposant shapenig parameter for tds_mgtwr model, default 6.
-#' @return A vector of predictions if beta_proj is FALSE or a list with a vector named Y_predicted and a matrix named Beta_proj_out.
-#' @details if method_pred ='tWtp_model',  the weighting matrix for prediction is
-#' based on the expected weights of outsample data if they were had been added to
-#' insample data to estimate the corresponding MGWRSAR (see Geniaux 2022 for
-#' further detail), if method_pred ='shepard'a shepard kernel with k_extra neighbours (default 8) is used and if method_pred='kernel_model' the same kernel
-#' and number of neighbors as for computing the MGWRSAR model is used.
-#' @seealso  MGWRSAR, golden_search_bandwidth, plot_mgwrsar, kernel_matW
-#' @examples
-#' \donttest{
-#' library(mgwrsar)
-#'  data(mydata)
-#'  coords=as.matrix(mydata[,c("x","y")])
-#' length_out=800
-#' index_in=sample(1:1000,length_out)
-#' index_out=(1:1000)[-index_in]
-#'
-#' model_GWR_insample<-MGWRSAR(formula = 'Y_gwr~X1+X2+X3', data = mydata[index_in,],
-#' coords=coords[index_in,],fixed_vars=NULL,kernels=c ('gauss'),H=8, Model = 'GWR',
-#' control=list(adaptive=TRUE))
-#' summary(model_GWR_insample)
-#'
-#' newdata=mydata[index_out,]
-#' newdata_coords=coords[index_out,]
-#' newdata$Y_mgwrsar_1_0_kv=0
-#'
-#' Y_pred=predict(model_GWR_insample, newdata=newdata,
-#' newdata_coords=newdata_coords)
-#' head(Y_pred)
-#' head(mydata$Y_gwr[index_out])
-#' sqrt(mean((mydata$Y_gwr[index_out]-Y_pred)^2)) # RMSE
-#' }
-#' @noRd
 predict_mgwrsar  <- function(model, newdata, newdata_coords, W = NULL, type = "BPN", h_w = 100,kernel_w = "rectangle",maxobs=4000,beta_proj=FALSE,method_pred='TP', k_extra = 8,exposant=8) {
   ##############
   ##### WARNINGS
   ##############
-  # if(!(method_pred %in% c('Multiscale_GDT','shepard','tWtp_model')) & model@Model %in% c('multiscale_gwr','tds_mgwr','atds_gwr','atds_mgwr')) {
-  #   cat("Warning: only shepard and tWtp_model method for method_pred can be used with multiscale_gwr and tds_mgwr \n automatic switch to method_pred='tWtp_model'")
-  #   method_pred='tWtp_model'
-  # }
 
   if(is.null(model@TP) | length(model@TP)==nrow(model@X)) noTP=TRUE else noTP=FALSE
 
@@ -70,7 +13,7 @@ if(!noTP & method_pred=='TP') {
 
 if(!is(model,'mgwrsar')) stop('A mgwrsar class object is required')
 
-  tp_supported <- model@Model %in% c("GWR","MGWR")  # + autres si tu les supportes vraiment
+  tp_supported <- model@Model %in% c("GWR","MGWR")  # + other models if truly supported
 
   if (method_pred == "TP" && !tp_supported) {
     warning("method_pred='TP' is only implemented for ... Switching to 'tWtp_model'.")
@@ -110,7 +53,50 @@ if(is.null(newdata) | is.null(newdata_coords)) stop('provide the newdata and new
 }
   insample_size=length(model@Y)
   outsample_size=nrow(newdata)
+  m=insample_size
+  n=outsample_size+insample_size
+  S=1:m
+  O=(m+1):(n)
+  YS = model@Y
+  e=model@residuals
+  namesX=colnames(model@X)
 
+  get_exact_matches_edges <- function(coords, S, O, digits = NULL) {
+    coords <- as.data.frame(coords)
+    d <- ncol(coords)
+    if (!d %in% c(2, 3))
+      stop("coords must have 2 or 3 columns")
+
+    # Optional: rounding for numerical stability
+    if (!is.null(digits)) {
+      coords[S, 1:d] <- round(coords[S, 1:d], digits)
+      coords[O, 1:d] <- round(coords[O, 1:d], digits)
+    }
+
+    # Keys
+    keyS <- do.call(paste, c(coords[S, 1:d, drop = FALSE], sep = "||"))
+    keyO <- do.call(paste, c(coords[O, 1:d, drop = FALSE], sep = "||"))
+
+    # For each key in S, store the corresponding i indices
+    mapS <- split(S, keyS)
+
+    # Filter j values that have at least one match
+    has_match <- keyO %in% names(mapS)
+    if (!any(has_match)) {
+      return(list(j = integer(0), i = integer(0)))
+    }
+
+    # Construction edge list
+    j_rep <- O[has_match]
+    i_list <- mapS[keyO[has_match]]
+
+    i_idx <- unlist(i_list, use.names = FALSE)
+    j_idx <- rep(j_rep, times = lengths(i_list))
+
+    list(j = j_idx, i = i_idx)
+  }
+
+  
 
   if(model@Type=='GDT') {
 
@@ -135,16 +121,20 @@ if(is.null(newdata) | is.null(newdata_coords)) stop('provide the newdata and new
       coords=rbind(model@coords,newdata_coords[,1:2])
     }
   coords<-make_unique_by_structure(coords)
-  m=insample_size
-  n=outsample_size+insample_size
-  S=1:m
-  O=(m+1):(n)
-  YS = model@Y
-  e=model@residuals
-  namesX=colnames(model@X)
 
 
-  if(is.null(W) & !(model@Model %in% c('OLS','GWR','GWR_glmboost','MGWR','multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr'))) {
+  edges <- get_exact_matches_edges(coords, S, O, digits = 6)
+  j_abs <- unique(edges$j)
+  j_pos <- match(j_abs, O)
+  row_idx <- match(edges$j, O)
+  col_idx <- match(edges$i, S)
+  ok <- !is.na(row_idx) & !is.na(col_idx)
+  row_idx <- row_idx[ok]
+  col_idx <- col_idx[ok]
+  j_pos <- unique(row_idx)
+
+
+  if(is.null(W) & !(model@Model %in% c('OLS','GWR','GWR_glmboost','MGWR','multiscale_gwr','tds_mgwr','tds_mgtwr','atds_mgwr','atds_gwr'))) {
     if(!is.null(model@h_w)){
       h_w=model@h_w
       kernel_w=model@kernel_w
@@ -228,7 +218,7 @@ if(model@Model=='OLS') {
   beta_in=cbind(model@Betav,matrix(model@Betac,nrow=m,ncol=length(model@Betac),byrow=TRUE))
   colnames(beta_in)=c(colnames(model@Betav),names(model@Betac))
 
-} else if(model@Model %in% c('multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr')){
+} else if(model@Model %in% c('multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr','tds_mgtwr')){
 beta_in=model@Betav
 
 } else if(model@Model=='MGWRSAR_0_0_kv'){
@@ -253,10 +243,10 @@ beta_in=model@Betav
 if(!(model@Model %in% c('OLS','SAR'))) tokeep=which(!is.na(beta_in[,1]))
 
 ##############
-##### calcul W_extra et Beta_proj_out
+##### compute W_extra and Beta_proj_out
 ##############
 
-if(model@Model %in% c('multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr')){
+if(model@Model %in% c('multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr','tds_mgtwr')){
   if(method_pred!='tWtp_model') res=prep_d(coords,NN=model@NN,TP=O,extrapol=TRUE,ratio=1,QP=S,kernels=model@kernels,Type=model@Type)
   if(method_pred!='shepard'){
   Beta_proj_out=matrix(0,ncol=ncol(beta_in),nrow=length(O))
@@ -264,9 +254,9 @@ if(model@Model %in% c('multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr')){
   for(k in colnames(beta_in)){ #parallel ?
     #cat(k,' ')
       if(method_pred=='tWtp_model'){
-        #browser()
+
         if(length(model@Ht)>0) myH=c(model@H[k],model@Ht[k]) else myH=model@H[k]
-        W_extra=kernel_matW(H=myH,kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=S,Type=model@Type,dists=res$dists,diagnull=FALSE,extrapol=F,alpha=model@alpha)[,O]
+        W_extra=kernel_matW(H=myH,kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=S,Type=model@Type,dists=res$dists,diagnull=FALSE,extrapol=FALSE,alpha=model@alpha)[,O]
         # W_extra = t(apply(W_extra, 1, function(x) x * as.numeric(x >= sort(x, decreasing = T)[k_extra + 2])))
         W_extra<-W_extra[tokeep,]
       } else if(method_pred=='model'){
@@ -278,25 +268,42 @@ if(model@Model %in% c('multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr')){
             if(!exists('resS')){
               resS=prep_d(coords[c(S,O),1:2],NN=model@NN,TP=O,extrapol=TRUE,ratio=1,QP=S,kernels=model@kernels[1],Type='GD')
             }
-            W_extra=kernel_matW(H=myH[1],kernels=model@kernels[1],coords=coords[c(S,O),1:2],NN=model@NN,TP=O,Type='GD',adaptive=model@adaptive[1],indexG=resS$indexG,dists=resS$dists,diagnull=FALSE,extrapol=T,QP=S,alpha=model@alpha)
+            W_extra=kernel_matW(H=myH[1],kernels=model@kernels[1],coords=coords[c(S,O),1:2],NN=model@NN,TP=O,Type='GD',adaptive=model@adaptive[1],indexG=resS$indexG,dists=resS$dists,diagnull=FALSE,extrapol=TRUE,QP=S,alpha=model@alpha)
           } else if(myH[1]==model@V[1]) {
             if(!exists('resT')){
               resT=prep_d(as.matrix(coords[c(S,O),3],ncol=1),NN=model@NN,TP=O,extrapol=TRUE,ratio=1,QP=S,kernels=model@kernels[2],Type='T')
             }
-            W_extra=kernel_matW(H=myH[2],kernels=model@kernels[2],coords=as.matrix(coords[c(S,O),3],ncol=2),NN=model@NN,TP=O,Type='T',adaptive=F,indexG=resT$indexG,dists=resT$dists,diagnull=FALSE,extrapol=T,QP=S,alpha=model@alpha)
+            W_extra=kernel_matW(H=myH[2],kernels=model@kernels[2],coords=as.matrix(coords[c(S,O),3],ncol=2),NN=model@NN,TP=O,Type='T',adaptive=FALSE,indexG=resT$indexG,dists=resT$dists,diagnull=FALSE,extrapol=TRUE,QP=S,alpha=model@alpha)
             } else {
 
-              W_extra=kernel_matW(H=myH,kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,indexG=res$indexG,dists=res$dists,diagnull=FALSE,extrapol=T,QP=S,alpha=model@alpha)
+              W_extra=kernel_matW(H=myH,kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,indexG=res$indexG,dists=res$dists,diagnull=FALSE,extrapol=TRUE,QP=S,alpha=model@alpha)
             }
-         # W_extra=kernel_matW(H=myH,kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,indexG=res$indexG,dists=res$dists,diagnull=FALSE,extrapol=T,QP=S,alpha=model@alpha) # visiblement c'est mieux en pure GDT pour les prédictions
+         # W_extra=kernel_matW(H=myH,kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,indexG=res$indexG,dists=res$dists,diagnull=FALSE,extrapol=TRUE,QP=S,alpha=model@alpha) # apparently pure GDT works better for predictions
         } else {
           myH=model@H[k]
-          if(myH[1]==model@V[1])  W_extra=matrix(1,nrow=length(O),ncol=length(S)) else W_extra=kernel_matW(H=myH,kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,indexG=res$indexG,dists=res$dists,diagnull=FALSE,extrapol=T,QP=S,alpha=model@alpha)
+          if(myH[1]==model@V[1])  W_extra=matrix(1,nrow=length(O),ncol=length(S)) else W_extra=kernel_matW(H=myH,kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,indexG=res$indexG,dists=res$dists,diagnull=FALSE,extrapol=TRUE,QP=S,alpha=model@alpha)
         }
         #if(model@Type=='GDT') W_extra = t(apply(W_extra, 1, function(x) x * as.numeric(x >= sort(x, decreasing = T)[k_extra + 2])))
         W_extra<-normW(W_extra^exposant)
       }
-      Beta_proj_out[,k]=as.matrix(W_extra %*% beta_in[tokeep,k])
+    # 1) reset to zero all rows j affected by duplicates
+    if(length(j_pos)>0){
+      W_extra[j_pos, ] <- 0
+      W_extra[cbind(row_idx, col_idx)] <- 1
+      # --- after computing W_extra ---
+      if (length(j_pos) > 0 && length(row_idx) > 0) {
+
+        # 1) set all affected rows to 0
+        W_extra[j_pos, ] <- 0
+
+        # 2) set exact-match weights to 1
+        W_extra[cbind(row_idx, col_idx)] <- 1
+
+        # 3) normalize ONLY these rows (equal weights if multiple i)
+        W_extra<-normW(W_extra)
+      }
+    }
+    Beta_proj_out[,k]=as.matrix(W_extra %*% beta_in[tokeep,k])
   }
   if(exists('res')) rm(res)
   if(exists('resS')) rm(resS)
@@ -304,11 +311,29 @@ if(model@Model %in% c('multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr')){
   gc()
   } else if(method_pred=='shepard'){
     if(model@Type=='GD'){
-      W_extra=kernel_matW(H=k_extra,kernels='shepard',coords=coords[c(S,O),],NN=k_extra+1,TP=O,Type=model@Type,adaptive=F,diagnull=FALSE,extrapol=T,QP=S)
+      W_extra=kernel_matW(H=k_extra,kernels='shepard',coords=coords[c(S,O),],NN=k_extra+1,TP=O,Type=model@Type,adaptive=FALSE,diagnull=FALSE,extrapol=TRUE,QP=S)
     } else {
       range01 <- function(x){(x-min(x))/(max(x)-min(x))};
       coords2=apply(coords,2,range01)
-      W_extra=kernel_matW(H=k_extra,kernels='shepard',coords=coords2[c(S,O),],NN=k_extra+1,TP=O,Type='GD',adaptive=F,diagnull=FALSE,extrapol=T,QP=S)
+      W_extra=kernel_matW(H=k_extra,kernels='shepard',coords=coords2[c(S,O),],NN=k_extra+1,TP=O,Type='GD',adaptive=FALSE,diagnull=FALSE,extrapol=TRUE,QP=S)
+    }
+    # 1) reset to zero all rows j affected by duplicates
+    if(length(j_pos)>0){
+      W_extra[j_pos, ] <- 0
+      W_extra[cbind(row_idx, col_idx)] <- 1
+      # --- after computing W_extra ---
+      if (length(j_pos) > 0 && length(row_idx) > 0) {
+
+        # 1) set all affected rows to 0
+        W_extra[j_pos, ] <- 0
+
+        # 2) set exact-match weights to 1
+        W_extra[cbind(row_idx, col_idx)] <- 1
+
+        # 3) normalize ONLY these rows (equal weights if multiple i)
+        W_extra<-normW(W_extra)
+
+      }
     }
     Beta_proj_out=as.matrix(W_extra %*% beta_in[tokeep,])
   }
@@ -316,38 +341,57 @@ if(model@Model %in% c('multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr')){
 
   if(method_pred=='tWtp_model'){
     if(model@Type=='GD') {
-      W_extra=kernel_matW(H=c(model@H,model@Ht),kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=S,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=F)[,O]
+      W_extra=kernel_matW(H=c(model@H,model@Ht),kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=S,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=FALSE)[,O]
       W_extra<-W_extra[tokeep,]
       W_extra<- normW(Matrix::t(W_extra))
     } else if(model@Type=='GDT'){
-        W_extra=kernel_matW(H=c(model@H,model@Ht),kernels=model@kernels,coords=coords,NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=T,QP=S)
+        W_extra=kernel_matW(H=c(model@H,model@Ht),kernels=model@kernels,coords=coords,NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=TRUE,QP=S)
         W_extra<-W_extra^exposant
         W_extra<-normW(W_extra)
       }
   } else if(method_pred=='model'){
     if(model@Type=='GD') {
-      W_extra=kernel_matW(H=c(model@H,model@Ht),kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=T,QP=S)
+      W_extra=kernel_matW(H=c(model@H,model@Ht),kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=TRUE,QP=S)
   } else if(model@Type=='GDT') {
-      W_extra=kernel_matW(H=c(model@H,model@Ht),kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=T,alpha=model@alpha,QP=S)
+      W_extra=kernel_matW(H=c(model@H,model@Ht),kernels=model@kernels,coords=coords[c(S,O),],NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=TRUE,alpha=model@alpha,QP=S)
       #W_extra=t(apply(W_extra,1,function(x) x*as.numeric(x>=sort(x,decreasing = T)[k_extra+2])))
       W_extra<-W_extra[,tokeep]
       W_extra<-W_extra^exposant
       W_extra<-normW(W_extra)
   } else if(model@Type=='T'){
-    W_extra=kernel_matW(H=c(model@Ht),kernels=model@kernels,coords=as.matrix(coords[c(S,O)],ncol=1),NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=T,QP=S)
+    W_extra=kernel_matW(H=c(model@Ht),kernels=model@kernels,coords=as.matrix(coords[c(S,O)],ncol=1),NN=model@NN,TP=O,Type=model@Type,adaptive=model@adaptive,diagnull=FALSE,extrapol=TRUE,QP=S)
     }
   } else {
       kernels_pred='shepard'
       adaptive_pred=FALSE
       NNN=k_extra+1
-      W_extra=kernel_matW(H=c(k_extra),kernels=kernels_pred,coords=coords,NN=NNN,TP=O,Type=model@Type,adaptive=adaptive_pred,diagnull=FALSE,extrapol=T,QP=S)
+      W_extra=kernel_matW(H=c(k_extra),kernels=kernels_pred,coords=coords,NN=NNN,TP=O,Type=model@Type,adaptive=adaptive_pred,diagnull=FALSE,extrapol=TRUE,QP=S)
     }
 }
 
-if(!(model@Model %in% c('OLS','SAR','multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr')) ) Beta_proj_out=as.matrix(W_extra %*% beta_in[tokeep,])
+if(!(model@Model %in% c('OLS','SAR','multiscale_gwr','tds_mgwr','atds_mgwr','atds_gwr','tds_mgtwr')) ) {
+  # 1) reset to zero all rows j affected by duplicates
+  if(length(j_pos)>0){
+    W_extra[j_pos, ] <- 0
+    W_extra[cbind(row_idx, col_idx)] <- 1
+    # --- after computing W_extra ---
+    if (length(j_pos) > 0 && length(row_idx) > 0) {
+
+      # 1) set all affected rows to 0
+      W_extra[j_pos, ] <- 0
+
+      # 2) set exact-match weights to 1
+      W_extra[cbind(row_idx, col_idx)] <- 1
+
+      # 3) normalize ONLY these rows (equal weights if multiple i)
+      W_extra<-normW(W_extra)
+    }
+  }
+  Beta_proj_out=as.matrix(W_extra %*% beta_in[tokeep,])
+}
 
 ##############
-##### calcul Y_predicted
+##### compute Y_predicted
 ##############
 
 if(model@Model=='SAR') {
@@ -357,7 +401,6 @@ if(model@Model=='SAR') {
   Y_predicted=BP_pred_SAR(YS,X=as.matrix(rbind(model@X,newdata[,namesX])),W,e,beta_hat=rbind(beta_in,as.matrix(Beta_proj_out)),lambda_hat=c(lambda_in,as.numeric(W_extra %*%lambda_in)),S,O,type,k_extra=k_extra,kernel_extra=kernel_extra,model=model@Model, W_extra = W_extra,coords=coords,maxobs=maxobs)
 } else {
   ## ALL VARYING COEF MODEL without spatial autocorrelation
-  #browser()
   X=as.matrix(newdata[,namesX])
   Y_predicted=rowSums(as.matrix(Beta_proj_out*X))
 }
