@@ -30,7 +30,7 @@
 #'
 #' Gao, S., Mei, C. L., Xu, Q. X., & Wang, N. (2021). Non-iterative multiscale estimation for spatial autoregressive geographically weighted regression models. \emph{Entropy}, 25(2), 320.
 #'
-#' Geniaux, G. (2025). Top-Down Scale Approaches for Multiscale GWR with Locally Adaptive Bandwidths. \emph{Springer Nature}, 2021 LATEX template.
+#' Geniaux, G. (2026). Top-down scale approaches for multiscale gwr with locally adaptive bandwidths. \emph{Journal of Geographical Systems}: 1–50.
 #'
 #' @examples
 #' \donttest{
@@ -54,19 +54,43 @@
 #'               top = textGrob("DGP Geniaux (2024)", gp = gpar(fontsize = 20, font = 3)))
 #' }
 #' @export
-simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NULL, nuls = NULL, lambda = NULL, NN = 4, config_beta = 'default', config_snr = 0.7, config_eps = 'normal') {
+simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NULL,
+                            nuls = NULL, lambda = NULL, NN = 4, config_beta = 'default',
+                            config_snr = 0.7, config_eps = 'normal') {
 
-  # 1. RNG Initialization (Fixed Seed)
+  # 1. RNG Initialization
+  # Keep L'Ecuyer for parallel/OS portability of runif/rnorm
   set.seed(myseed, kind = "L'Ecuyer-CMRG", normal.kind = "Inversion")
 
   W <- NULL
 
-  # --- DETERMINISTIC SNR FUNCTION ---
-  get_snr_fixed <- function(k, config_snr, XB, eps_raw) {
-    eps_scaled <- k * eps_raw
-    # Inverse SNR formula: finding k such that signal/total ratio matches config_snr
-    # (Note: ensure this formula is indeed the one you intend to keep)
-    abs(config_snr - (sum(XB^2)) / sum((XB + eps_scaled)^2))
+  # --- HELPER: ANALYTICAL SNR SOLVER ---
+  # (Still needed to avoid numerical optimization discrepancies later)
+  get_k_analytical <- function(target_snr, XB, eps_raw) {
+    sum_XB2 <- sum(XB^2)
+    sum_eps2 <- sum(eps_raw^2)
+    sum_XB_eps <- sum(XB * eps_raw)
+
+    a <- target_snr * sum_eps2
+    b <- target_snr * 2 * sum_XB_eps
+    c <- target_snr * sum_XB2 - sum_XB2
+
+    delta <- b^2 - 4 * a * c
+    if (delta < 0) return(1)
+
+    k1 <- (-b + sqrt(delta)) / (2 * a)
+    k2 <- (-b - sqrt(delta)) / (2 * a)
+    if (k1 > 0) return(k1) else return(abs(k2))
+  }
+
+  # --- HELPER: ROBUST SAMPLE ---
+  # Replaces sample(x:y, size, replace=TRUE) to guarantee Linux/Mac consistency
+  # regardless of R version (before/after 3.6.0).
+  robust_sample_int <- function(min_val, max_val, size) {
+    # runif generates between min and max (often exclusive), ceiling rounds to integers
+    # This is the mathematical equivalent of uniform sampling with replacement
+    return(ceiling(runif(size, min_val - 1, max_val)))
+    # Note: runif(n, 0, 365) -> (0, 365). Ceiling -> 1..365.
   }
 
   # 2. Model Type Configuration
@@ -108,10 +132,9 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
 
   X1 <- runif(n, -sqrt(3), sqrt(3))
   X2 <- rnorm(n)
-  X3 <- rnorm(n) # Default (will be overwritten or used depending on case)
+  X3 <- rnorm(n)
 
-  # --- RAW NOISE GENERATION (FROZEN NOISE) ---
-  # Done HERE, once, to ensure Mac/Linux identity
+  # --- RAW NOISE GENERATION ---
   if (config_eps == 'normal') {
     eps_raw <- rnorm(n, 0, 1)
   } else if (config_eps == 'unif') {
@@ -155,8 +178,12 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
     Beta2 <- Beta4 <- Beta3 <- 4 * x * sin(sqrt((6^2 * (-0.4 + x * 2)^4)))
 
   } else if (type == 'spatiotemp') {
-    time <- sample(1:365, n, replace = TRUE)
-    year <- sample(1:4, n, replace = TRUE)
+    # --- CRITICAL FIX: Using robust_sample_int ---
+    # Replaces: time <- sample(1:365, n, replace = TRUE)
+    time <- robust_sample_int(1, 365, n)
+    # Replaces: year <- sample(1:4, n, replace = TRUE)
+    year <- robust_sample_int(1, 4, n)
+
     x_cor <- x + abs(time - 210) / 210
     x_cor <- x_cor / max(x_cor)
     y_cor <- y + abs(time - 210) / 210
@@ -179,8 +206,9 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
     Beta4 <- 4 * sin(sqrt((6^3 * (-x - y)^2))) + 2 * (x - 0.5)
 
   } else if (type == 'spatiotemp_fake') {
-    time <- sample(1:365, n, replace = TRUE)
-    year <- sample(1:4, n, replace = TRUE)
+    # --- CRITICAL FIX ---
+    time <- robust_sample_int(1, 365, n)
+    year <- robust_sample_int(1, 4, n)
 
     Beta1 <- 3 * (x + y) - 1
     Beta2 <- (84 * (x * y) * (1 - x) * (1 - y))
@@ -188,31 +216,29 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
     Beta4 <- 4 * sin(sqrt((6^3 * (-x - y)^2))) + 2 * (x - 0.5)
 
   } else if (type == 'same_ht') {
-    time <- sample(1:365, n, replace = TRUE)
-    year <- sample(1:4, n, replace = TRUE)
+    # --- CRITICAL FIX ---
+    time <- robust_sample_int(1, 365, n)
+    year <- robust_sample_int(1, 4, n)
 
     x_cor <- x + abs(time - 210) / 210
     x_cor <- x_cor / max(x_cor)
     y_cor <- y + abs(time - 210) / 210
     y_cor <- (y_cor - min(y_cor)) / max(y_cor - min(y_cor))
 
-    z1 <- x_cor - 0.2
-    z2 <- y_cor - 0.2
-    z3 <- x_cor + 0.2
-    z4 <- y_cor + 0.2
+    z1 <- x_cor - 0.2; z2 <- y_cor - 0.2
+    z3 <- x_cor + 0.2; z4 <- y_cor + 0.2
     Beta1 <- 84 * (z1 * z2) * (1 - z1) * (1 - z2)
     Beta2 <- 84 * (z3 * z4) * (1 - z3) * (1 - z4)
     Beta3 <- 84 * (z1 * z4) * (1 - z1) * (1 - z4)
     Beta4 <- 84 * (z3 * z2) * (1 - z3) * (1 - z2)
 
   } else if (type == 'same_h') {
-    time <- sample(1:365, n, replace = TRUE)
-    year <- sample(1:4, n, replace = TRUE)
+    # --- CRITICAL FIX ---
+    time <- robust_sample_int(1, 365, n)
+    year <- robust_sample_int(1, 4, n)
 
-    z1 <- x - 0.2
-    z2 <- y - 0.2
-    z3 <- x + 0.2
-    z4 <- y + 0.2
+    z1 <- x - 0.2; z2 <- y - 0.2
+    z3 <- x + 0.2; z4 <- y + 0.2
     Beta1 <- 84 * (z1 * z2) * (1 - z1) * (1 - z2)
     Beta2 <- 84 * (z3 * z4) * (1 - z3) * (1 - z4)
     Beta3 <- 84 * (z1 * z4) * (1 - z1) * (1 - z4)
@@ -231,7 +257,7 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
     if (constant == 3) Beta3 = 3
     if (constant == 4) Beta4 = 3
   } else {
-    for (i in 1:length(constant)) assign(paste0('Beta', constant[i]), 3)
+    for (i in seq_along(constant)) assign(paste0('Beta', constant[i]), 3)
   }
 
   # Null Coefficients Handling
@@ -241,27 +267,25 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
     if (nuls == 3) Beta3 = 0
     if (nuls == 4) Beta4 = 0
   } else {
-    for (i in 1:length(nuls)) assign(paste0('Beta', nuls[i]), 0)
+    for (i in seq_along(nuls)) assign(paste0('Beta', nuls[i]), 0)
   }
 
   # 5. Construction of Y and mydata (Robustification)
   if (type %in% c('FT2017', 'Gao2021')) {
     XB <- Beta1 + Beta2 * X1 + Beta3 * X2
 
-    # SNR Optimization with frozen noise
-    k <- (optimize(get_snr_fixed, lower = 1, upper = 100, config_snr = config_snr, XB = XB, eps_raw = eps_raw))$minimum
-    eps <- k * eps_raw
+    k <- get_k_analytical(config_snr, XB, eps_raw)
 
+    eps <- k * eps_raw
     Y <- XB + eps
     if (!is.null(iW)) Y <- as.numeric(iW %*% Y)
     mydata <- data.frame(Y, X1, X2, Beta1, Beta2, Beta3, eps, x, y)
 
   } else if (type %in% c('GG2024_univ')) {
-    # For this specific case, you had a different noise logic
     if (config_snr >= 0.9) eps <- rnorm(n, 0, 0.1)
     else if (config_snr > 0.7) eps <- rnorm(n, 0, 0.25)
     else if (config_snr > 0.5) eps <- rnorm(n, 0, 0.5)
-    else eps <- eps_raw # Fallback
+    else eps <- eps_raw
 
     Y <- Beta1 + Beta2 * X1 + Beta3 * X2 + Beta4 * X3 + eps
     Y1 <- Beta1 + eps
@@ -273,7 +297,7 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
 
   } else if (type == 'GG2024_2') {
     XB <- Beta1 + Beta2 * X1
-    k <- (optimize(get_snr_fixed, lower = 1, upper = 100, config_snr = config_snr, XB = XB, eps_raw = eps_raw))$minimum
+    k <- get_k_analytical(config_snr, XB, eps_raw)
     eps <- k * eps_raw
     Y <- XB + eps
     mydata <- data.frame(Y, X1, Beta1, Beta2, eps, x, y)
@@ -288,14 +312,14 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
     Beta7 <- 4 * sin(sqrt(12 * (x - 0.5)^2 + 12 * (y - 0.5)^2))
     Beta8 <- 84 * (x * y) * (1 - x) * (1 - y)
     XB <- Beta1 + Beta2 * X1 + Beta3 * X2 + Beta4 * X3 + Beta5 * X4 + Beta6 * X5 + Beta7 * X6 + Beta8 * X7
-    k <- (optimize(get_snr_fixed, lower = 1, upper = 100, config_snr = config_snr, XB = XB, eps_raw = eps_raw))$minimum
+
+    k <- get_k_analytical(config_snr, XB, eps_raw)
     eps <- k * eps_raw
     Y <- XB + eps
     if (!is.null(iW)) Y <- as.numeric(iW %*% Y)
     mydata <- data.frame(Y, X1, X2, X3, X4, X5, X6, X7, Beta1, Beta2, Beta3, Beta4, Beta5, Beta6, Beta7, Beta8, eps, x, y)
 
   } else if (type == 'GG2024_16') {
-    # (Creation of X4 to X15...)
     X4 <- runif(n, -sqrt(3), sqrt(3)); X5 <- rnorm(n); X6 <- runif(n, -sqrt(3), sqrt(3)); X7 <- rnorm(n)
     X8 <- runif(n, -sqrt(3), sqrt(3)); X9 <- rnorm(n); X10 <- runif(n, -sqrt(3), sqrt(3)); X11 <- rnorm(n)
     X12 <- runif(n, -sqrt(3), sqrt(3)); X13 <- rnorm(n); X14 <- runif(n, -sqrt(3), sqrt(3)); X15 <- rnorm(n)
@@ -304,23 +328,25 @@ simu_multiscale <- function(n = 1000, myseed = 1, type = 'GG2024', constant = NU
     Beta6 <- 1 + 1 / 324 * (36 - (6 - x * 25 / 2)^2) * (36 - (6 - y * 25 / 2)^2)
     Beta7 <- 4 * sin(sqrt(12 * (x - 0.5)^2 + 12 * (y - 0.5)^2))
     Beta8 <- 84 * (x * y) * (1 - x) * (1 - y)
-    Beta9 <- Beta1; Beta10 <- Beta2; Beta11 <- Beta3; Beta12 <- Beta4
-    Beta12 <- Beta4 <- 3 # Remove as in your original code
+    Beta9 <- Beta1; Beta10 <- Beta2; Beta11 <- Beta3;
+    Beta12 <- 3
     Beta13 <- Beta5; Beta14 <- Beta6; Beta15 <- Beta7; Beta16 <- Beta8
 
     XB <- Beta1 + Beta2 * X1 + Beta3 * X2 + Beta4 * X3 + Beta5 * X4 + Beta6 * X5 + Beta7 * X6 + Beta8 * X7 +
       Beta9 * X8 + Beta10 * X9 + Beta11 * X10 + Beta12 * X11 + Beta13 * X12 + Beta14 * X13 + Beta15 * X14 + Beta16 * X15
 
-    k <- (optimize(get_snr_fixed, lower = 1, upper = 100, config_snr = config_snr, XB = XB, eps_raw = eps_raw))$minimum
+    k <- get_k_analytical(config_snr, XB, eps_raw)
     eps <- k * eps_raw
     Y <- XB + eps
     if (!is.null(iW)) Y <- as.numeric(iW %*% Y)
-    mydata <- data.frame(Y, X1, X2, X3, X4, X5, X6, X7, X8, X9, X10, X11, X12, X13, X14, X15, Beta1, Beta2, Beta3, Beta4, Beta5, Beta6, Beta7, Beta8, Beta9, Beta10, Beta11, Beta12, Beta13, Beta14, Beta15, Beta16, eps, x, y)
+    mydata <- data.frame(Y, X1, X2, X3, X4, X5, X6, X7, X8, X9, X10, X11, X12, X13, X14, X15,
+                         Beta1, Beta2, Beta3, Beta4, Beta5, Beta6, Beta7, Beta8,
+                         Beta9, Beta10, Beta11, Beta12, Beta13, Beta14, Beta15, Beta16, eps, x, y)
 
   } else {
     # Default case
     XB <- Beta1 + Beta2 * X1 + Beta3 * X2 + Beta4 * X3
-    k <- (optimize(get_snr_fixed, lower = 1, upper = 100, config_snr = config_snr, XB = XB, eps_raw = eps_raw))$minimum
+    k <- get_k_analytical(config_snr, XB, eps_raw)
     eps <- k * eps_raw
     Y <- XB + eps
     if (!is.null(iW)) Y <- as.numeric(iW %*% Y)
